#!/bin/sh

DEST=$PWD/voyage-install

#KERNEL_VER=2.6.8-486-voyage
KERNEL_VER=2.6.15-486-voyage

KERNEL_IMAGE=kernel-image-"$KERNEL_VER"
MODULE_IMAGE=madwifi-ng-modules-"$KERNEL_VER"

EXCLUDE="\
at,cpio,bsdmainutils,debconf-i18n,groff-base,ed,fdutils,info,libident,\
libpcre3,logrotate,libtext-charwidth-perl,libtext-iconv-perl,\
libtext-wrapi18n-perl,mailx,man-db,manpages,nano,nvi,syslinux,\
exim4,exim4-base,exim4-config,exim4-daemon-light,\
aptitude,tasksel,base-config,psmisc,liblocale-gettext-perl,\
libsigc++-1.2-5c102,console-data,console-data,console-tools,console-common,\
ipchains,apt-utils,libgcrypt7,libgcrypt11,libopencdk8,libgnutls10,libgnutls11,\
liblzo1,liblockfile1,libgdbm3,libgpg-error0,libtasn1-2,libtextwrap1,\
libconsole,libdb4.2"


INCLUDE="\
bridge-utils,debconf-english,dnsmasq,dhcp-client,elvis-tiny,\
iproute,libsysfs1,libatm1,logrotate,ntpdate,\
hotplug,ssh,pptpd,\
ebtables,ifenslave-2.6"

#EXCLUDE=""
#INCLUDE=""

REMOVE_PKGS=""

EXTRA_PKGS="\
hostap-utils,hostapd,wpasupplicant,\
pcmcia-cs,psmisc,\
watchdog,bzip2,\
nocatsplash,wireless-tools,\
madwifi-ng-tools,parprouted,\
less,traceroute\
"
#madwifi-tools,
#,nocatauth-gateway,\
#,madwifi-modules-$KERNEL_VER,\
#debfoster,deborphan,\
#quagga,wondershaper,openswan,\


MIRROR="ftp://ftp.hk.debian.org/debian"
#MIRROR="http://www.zentek-international.com/mirrors/debian"
#MIRROR="http://www.hk.debian.org/debian"

DIST="sarge"
#DIST="etch"

if [ -d $DEST ] ; then
	rm -rf $DEST
fi

BOOTSTRAP_OPT="--verbose"

MOUNT_PROC_SH=/usr/local/sbin/mount-proc.sh

export PATH=$PATH:/usr/sbin
export DEBIAN_FRONTEND=noninteractive


debootstrap --exclude=$EXCLUDE --include=$INCLUDE $DIST $DEST $MIRROR

RET=$?
if [ "$RET" -ne "0" ] ; then
        echo "debootstrap error.  Exit code = "$RET"."
        exit 1
fi

if [ ! -d $DEST ] ; then
	exit
fi

# Running post-installing scripts
echo "Running post-installing scripts ..."

###################################################################################
#### /etc/apt/source.list
####
gen_sourceslist()
{
cat <<EOF > $DEST/etc/apt/sources.list
deb ftp://ftp.hk.debian.org/debian/ sarge main
deb http://security.debian.org/ sarge/updates main
deb http://www.voyage.hk/dists/0.2 ./
EOF

chroot $DEST apt-get update
chroot $DEST apt-get -q=1 -y upgrade
}


#################################
# generate scripts
#################################
gen_scripts()
{
	export DEST
	./gen-remount.sh
}

#################################
#Remove extra package
#################################
remove_pkg()
{
	echo -n "Removing extra packages ... "
	REMOVE_PKGS=`echo $REMOVE_PKGS | sed "s/,/ /g"`
	for PKG in $REMOVE_PKGS
	do
		chroot $DEST apt-get -q -y remove $PKG
	done
	echo "Done"
}

#################################
# Setting hostname
#################################
set_hostname()
{
	HOST_NAME="voyage"
	echo "Setting hostname to voyage ..."
	echo "$HOST_NAME" > $DEST/etc/hostname
	#chroot $DEST hostname "$HOST_NAME"
}

#################################
# Setting Timezone
#################################
set_timezone()
{
	echo -n "Setting Timezone to GMT ... "
	
	TZ=GMT
	echo $TZ > $DEST/etc/timezone
	rm -f $DEST/etc/localtime
	ln -sf /usr/share/zoneinfo/$TZ $DEST/etc/localtime
	
	echo "Done"
}

#################################
# Copying scripts
#################################
copy_script()
{
	echo -n "Copying scripts ... "
	cp voyage.update $DEST/
	chmod +x $DEST/voyage.update
	
	cp lilo.conf.voyage $DEST/etc/
	
	cp voyage.conf $DEST/etc/
	
	cp voyage-configure.sh $DEST/usr/local/sbin
	
	cp motd.voyage $DEST/etc/
	cat $DEST/etc/motd.voyage >> $DEST/etc/motd 
	
	cp nat.sh $DEST/usr/local/sbin
	chmod +x $DEST/usr/local/sbin
	
	cp voyage-util $DEST/etc/init.d/
	chmod +x $DEST/etc/init.d/voyage-util 
	
	cp format-cf.sh $DEST/usr/local/sbin
	chmod +x $DEST/usr/local/sbin/format-cf.sh 
		
	echo "Done"
}

#################################
# Installing Kernel
#################################
install_kernel()
{
	if expr $KERNEL_VER : 2.6 >/dev/null ; then 
		chroot $DEST apt-get -q=2 -y install module-init-tools
	fi
	chroot $DEST $MOUNT_PROC_SH apt-get -q=2 -y install $KERNEL_IMAGE $MODULE_IMAGE
	
	chroot $DEST depmod -a $KERNEL_VER -F /boot/System.map-$KERNEL_VER
}

#################################
# Install Extra package
#################################
install_extra()
{
	echo -n "Installing extra packages ... "
	EXTRA_PKGS=`echo $EXTRA_PKGS | sed "s/,/ /g"`
	for PKG in $EXTRA_PKGS
	do
		chroot $DEST $MOUNT_PROC_SH apt-get -q=2 -y install $PKG
	done
	#chroot $DEST apt-get -q=2 -y install $EXTRA_PKGS
	echo "Done"
}

#################################
# Downloading Firmware from Prism54.org
#################################
install_prismfw()
{
	PRISM54_FIRMWARE_URL="http://jbnote.free.fr/prism54usb/data/firmwares/p54pci_1.0.4.3.arm"
	#PRISM54_FIRMWARE_URL="http://prism54.org/~mcgrof/firmware/1.0.4.3.arm"
	#PRISM54_FIRMWARE_URL="http://ruslug.rutgers.edu/~mcgrof/802.11g/firmware/1.0.4.3.arm"
	echo -n "Downloading Firmware from Prism54.org ... "
	chroot $DEST wget "$PRISM54_FIRMWARE_URL" \
		-O /usr/lib/hotplug/firmware/isl3890
	echo "Done"
}


#################################
# Downloading Firmware from Prism54.org
#################################
install_bcm43xxfw()
{
    BCM43XX_FIRMWARE_URL="http://openwrt.inf.fh-brs.de/~nbd/wl_apsta.o"

    echo -n "Downloading Firmware for bcm43xx ... "
    chroot $DEST wget "$BCM43XX_FIRMWARE_URL" \
        -O /usr/lib/hotplug/firmware/`basename $BCM43XX_FIRMWARE_URL`
    echo "Done"
}

#################################
# Installing lilo
#################################
install_lilo()
{
	chroot $DEST apt-get -q=2 -y install lilo
}

#################################
# Installing voyage-util
#################################
install_voyage_util()
{
	chroot $DEST update-rc.d -f ntpdate remove 
	chroot $DEST update-rc.d -f rmnologin remove 
	chroot $DEST update-rc.d voyage-util defaults 99
	touch $DEST/voyage.1st
}

###################################################################################
#### Generate voyage.dpkg-l, voyage.dpkg.list, voyage.depends.list
####
gen_dpkglist()
{
	chroot $DEST dpkg -l > $DEST/voyage.dpkg-l
	chroot $DEST dpkg-query -W --showformat='${Package} ${Version}\n' > $DEST/voyage.dpkg.list
	#cat $DEST/var/lib/dpkg/available | grep "^Package" | cut -f 2 -d " " | sort > $DEST/voyage.dpkg.list
	
	rm -f $DEST/voyage.depends.list
	for DPKG in $( find $DEST/var/cache/apt/archives/ -name "*.deb");	 
	do 
		dpkg -I $DPKG | egrep "^ Package|^ Depends"  | sed "s/^ Package : //g" >> $DEST/voyage.depends.list
	done
}

#################################
# Removing Locale
#################################
remove_locale()
{
	echo -n "Removing Locale in /usr/share/locale ... "
	find $DEST/usr/share/locale -name "*.mo" -exec rm '{}' ';'
	echo "Done"
}


#################################
# Removing Timezone
#################################
remove_timezone()
{
	echo -n "Removing Timezone in /usr/share/zoneinfo ... "
	rm -rf $DEST/usr/share/zoneinfo/posix
	rm -rf $DEST/usr/share/zoneinfo/right
	rm -rf $DEST/usr/share/zoneinfo/SystemV
	echo "Done"
}


#################################
# Removing Extra
#################################
remove_extra()
{
	echo -n "Zeoring available Package lists ... "
	chroot $DEST apt-get clean 
	for i in $(find $DEST/var/lib/apt/lists -type f \( -name \*Packages -o -name \*Sources \) 2>/dev/null); do  :>"$i"; done
	chroot $DEST dpkg --clear-avail
	rm -f $DEST/var/cache/apt/*.bin
	chroot $DEST apt-cache gencaches > /dev/null
	chroot $DEST remove.docs
	echo "Done"
}
		
#############################################################################

#################################
# Installing /ro and /rw directories
#################################
setup_rw()
{
	echo -n "Installing /ro and /rw directories ... "
	chroot $DEST mkdir /rw /ro ro/dev ro/etc ro/usr ro/var
	mv $DEST/root $DEST/ro/
	chroot $DEST ln -sf /rw/root /root
	mv $DEST/tmp $DEST/ro/
	chroot $DEST ln -sf /rw/tmp /tmp
	echo "Done"
	
	#################################
	# Setting up devices
	#################################
	echo -n "Setting up devices in /rw and /ro ... "
	DEVICES=`chroot $DEST ls /dev/ttyp*`
	DEVICES="$DEVICES `chroot $DEST ls /dev/ttyS*`"
	DEVICES="$DEVICES `chroot $DEST ls /dev/tty[0-9]`"
	DEVICES="$DEVICES `chroot $DEST ls /dev/tty1[0-2]`"
	DEVICES="$DEVICES `chroot $DEST ls /dev/tty`"
	DEVICES="$DEVICES `chroot $DEST ls /dev/pty*`"
	DEVICES="$DEVICES `chroot $DEST ls /dev/xconsole`"
	
	for DEVICE in $DEVICES
	do
	        if [ -c $DEVICE ]  ; then
	                chroot $DEST mv $DEVICE /ro/dev/
	                chroot $DEST ln -sf /rw$DEVICE $DEVICE
	        fi
	done
	                chroot $DEST mv /dev/xconsole /ro/dev/
	                chroot $DEST ln -sf /rw/dev/xconsole /dev/xconsole
	                chroot $DEST ln -sf /rw/dev/log  /dev/log
	echo "Done"
	
	#################################
	# Remapping files to /rw
	#################################
	echo -n "Remapping files to /rw ... "
	FILES="/etc/adjtime /etc/hostname /etc/issue /etc/issue.net /etc/resolv.conf /etc/mtab~"
	FILES=$FILES" /var/lock /var/log /var/mail /var/run /var/spool /var/tmp"
	
	for FILE in $FILES
	do
		if [ ! -f $DEST/$FILE ] ; then touch $DEST/$FILE ; fi
		chroot $DEST mv $FILE /ro/`dirname $FILE`
		chroot $DEST ln -sf /rw$FILE $FILE
	done
	echo "Done"	
	
	#################################
	# Modifying /etc/network/ifstate
	#################################
	echo -n "Modifying /etc/network/run/ifstate ... "
	#chroot $DEST ln -sf /rw/etc/network/ifstate /etc/network/ifstate
	#mkdir -p $DEST/ro/etc/network/
	#touch $DEST/ro/etc/network/ifstate

	chroot $DEST ln -sf /rw/etc/network/run/ifstate /etc/network/run/ifstate
	mkdir -p $DEST/ro/etc/network/run
	touch $DEST/ro/etc/network/run/ifstate
	echo "Done"
} 

setup_etc()
{
	#################################
	# Modifying /etc/init.d/syslogd
	#################################
	echo -n "Modifying /etc/init.d/syslogd ... "
	cat $DEST/etc/init.d/sysklogd | \
	sed -e 's/^SYSLOGD=\"\"/SYSLOGD\=\"\-p \/rw\/dev\/log \-m 0\"/' \
	>  $DEST/etc/init.d/sysklogd.tmp
	mv  $DEST/etc/init.d/sysklogd.tmp  $DEST/etc/init.d/sysklogd
	chmod +x $DEST/etc/init.d/sysklogd
	echo "Done"
	
	#################################
	# Modifying /etc/mtab
	#################################
	echo -n "Modifying /etc/mtab ... "
	chroot $DEST rm -f /etc/mtab
	chroot $DEST ln -s /proc/mounts /etc/mtab
	echo "Done"
	
	#################################
	# Modifying /etc/network/options
	#################################
	echo -n "Modifying /etc/network/options ... "
	cat $DEST/etc/network/options | sed -e "s/ip_forward=no/ip_forward=yes/" > $DEST/etc/network/options.tmp
	mv $DEST/etc/network/options.tmp $DEST/etc/network/options
	echo "Done"
	
	#################################
	# Modifying /etc/dnsmasq.conf
	#################################
	echo -n "Modifying /etc/dnsmasq.conf ... "
	DNSMASQ_CONF=/etc/dnsmasq.conf 
	DNSMASQ_MORE_CONF=/etc/dnsmasq.more.conf
	if [ -f $DEST/$DNSMASQ_CONF ] ; then
		echo "conf-file=$DNSMASQ_MORE_CONF" >> $DEST/$DNSMASQ_CONF
	fi	
	echo "dhcp-leasefile=/var/tmp/dnsmasq.leases" >> $DEST/$DNSMASQ_MORE_CONF
	echo "dhcp-range=wlan0,10.1.10.10,10.1.10.250,24h" >> $DEST/$DNSMASQ_MORE_CONF
	echo "dhcp-range=eth1,10.1.20.10,10.1.20.250,24h" >> $DEST/$DNSMASQ_MORE_CONF
	echo "dhcp-range=eth2,10.1.30.10,10.1.30.250,24h" >> $DEST/$DNSMASQ_MORE_CONF
	echo "dhcp-range=eth3,10.1.40.10,10.1.40.250,24h" >> $DEST/$DNSMASQ_MORE_CONF
	echo "Done"
		
	#################################
	# Adding configuration for watchdog
	#################################
	echo -n "Adding configuration for watchdog ... "
	WATCHDOG_CONF=$DEST/etc/watchdog.conf
	if [ -f $WATCHDOG_CONF ] ; then
		echo "watchdog-device = /dev/wd" >> $WATCHDOG_CONF
		echo "interval	    = 15" >> $WATCHDOG_CONF
		chroot $DEST mknod -m 600 /dev/wd c 10 130
	fi;
	echo "Done"
	
	#################################
	# Modifying /etc/init.d/mountall.sh
	#################################
	echo -n "Modifying /etc/init.d/mountall.sh ... "
	#echo "cp -a /ro/* /rw/" >> $DEST/etc/init.d/mountall.sh
	echo "tar -C /ro -cf - . | tar -C /rw -xf -  > /dev/null 2>&1" >> $DEST/etc/init.d/mountall.sh
	echo "Done"
	
	#################################
	# Modifying /etc/modules
	#################################
	echo -n "Modifying /etc/modules ... "
	echo "natsemi" >> $DEST/etc/modules
	echo "hostap_pci" >> $DEST/etc/modules
	echo "#ath_pci autocreate=none" >> $DEST/etc/modules
	echo "options ath_pci autocreate=none" > $DEST/etc/modprobe.d/madwifi
	echo "Done"	
}

gen_passwd()
{
	echo -n "Generating root password ... "
	cat $DEST/etc/passwd  | sed 's/^root::0/root:$1$0ljBNA5v$I8Nk72OF269GX04cB3HxX.:0/' > $DEST/etc/passwd
	echo " Done"
}

get_voyage_custom()
{
	echo "Getting voyage-custom from CVS ..."
	export CVSROOT=:pserver:punkytse@localhost:2401/var/lib/cvs	

	cd $DEST
	cvs export -r HEAD -d usr/share/voyage-custom voyage-custom
	cd $OLDPWD

	echo "Done"
}

gen_cd()
{
	echo "Generating bootable CD ..."
	echo "deb http://developer.linuxtag.net/knoppix ./" >> $DEST/etc/apt/sources.list
	#echo "deb http://debian.cn99.com/ubuntu hoary main multiverse restricted universe" >> $DEST/etc/apt/sources.list
	chroot $DEST apt-get update	
	chroot $DEST apt-get install hwsetup hwdata-knoppix
	chroot $DEST apt-get -q=2 -y install kernel-image-2.4.26
	#chroot $DEST apt-get -q=2 -y install linux-image-386
}

###############################################################################################

if [ "$1" == "cd" ] ; then
	gen_sourceslist
	gen_scripts
	remove_pkg
	set_hostname
	set_timezone
	#copy_script
	install_extra
	install_prismfw
	#install_bcm43xxfw
	gen_cd
	#gen_dpkglist
	remove_locale
	remove_extra
	remove_timezone
	setup_etc
else
	gen_sourceslist
	gen_scripts
	remove_pkg
	set_hostname
	set_timezone
	copy_script
	install_kernel
	install_extra
	install_prismfw
	#install_bcm43xxfw
	install_lilo
	install_voyage_util
	get_voyage_custom
	gen_passwd
	gen_dpkglist
	remove_locale
	remove_timezone
	remove_extra
	setup_rw
	setup_etc
fi                         
###############################################################################################
                           
##########################################################
echo ""                    
echo "Build completed "    
echo "Complete Voyage package is stored in:"
echo "	$DEST"             
echo ""
